﻿using System.Collections.Generic;
using System.Linq;
using NUnit.Framework;

namespace API.Tests
{
    [TestFixture]
    public class BugControllerTests
    {
        #region Example 2 - Get single bug
        [Test]
        public void GetBugByID_ValidID_ReturnsBugObject()
        {
            int expectedBugID = 1;

            //1) At this point we need to insert our bug via controller
            //InsertBug_ValidBugObject_ReturnsTrue
            
            Bug insertedBug = new Bug();
            insertedBug.ID = expectedBugID;
            controller.InsertBug(insertedBug);

            //4) Need to be able to get the bug via it's ID

            Bug actual = controller.GetBugByID(expectedBugID);
            Assert.AreEqual(insertedBug.ID, actual.ID);
        }

        [Test]
        public void InsertBug_ValidBugObject_ReturnsTrue()
        {
            Bug b = new Bug();
            bool result = controller.InsertBug(b);
            Assert.IsTrue(result);
        }

        //2) Implement - InsertBug + Contains
 
        //3) Refactor - move repository into ctor

        #endregion

        #region Example 3 - Get All Active Bugs

        [Test]
        public void GetAllActiveBugs_2BugsInserted_2BugsReturnedAsList()
        {
            Bug insertedBug = new Bug();
            insertedBug.ID = 1;
            insertedBug.IsActive = true;
            controller.InsertBug(insertedBug);

            Bug insertedBug2 = new Bug();
            insertedBug2.ID = 1;
            insertedBug2.IsActive = true;
            controller.InsertBug(insertedBug2);

            List<Bug> actualBugList = controller.GetAllActiveBugs();
            Assert.AreEqual(2, actualBugList.Count);
            Assert.IsTrue(actualBugList.Contains(insertedBug));
            Assert.IsTrue(actualBugList.Contains(insertedBug2));
        }

        //1) Create repository test


        #endregion

        #region Example 4 - Send Email After Inserting

        //Using setup methods to clean code
        private BugController controller;
        private StubEmailService stubEmailService;

        [SetUp]
        public void Setup()
        {
            stubEmailService = new StubEmailService();
            controller = new BugController(stubEmailService);
        }

        [Test]
        public void InsertBug_ValidBugObject_CallsEmailService_ToSendEmail()
        {
            controller.InsertBug(new Bug());
            //We need some kind of passing criteria for this test.  
            //How do we verify that the email was actually sent
            //Use a test double!

            Assert.AreEqual(1, stubEmailService.EmailsSent); //Checky property.
        }

        #endregion
    }
}
