﻿using System.Linq;
using NUnit.Framework;

namespace API.Tests
{
    [TestFixture]
    public class RepositoryTests
    {
        #region Example 1 - Insert Bug

        //1) Need a test to define how we insert a bug
        [Test]
        public void InsertBug_ValidBugObject_CountIncremented()
        {
            Repository repository = new Repository();
            repository.InsertBug(new Bug());
            //Assert.IsTrue(true, "Test Failed: Exception thrown when inserting"); 
                        //Not very readable
            //2) Adding a property to help us test
            Assert.AreEqual(1, repository.BugCount);
        }
        //3) Now implement the code (BugCount++);

        //)Refactor - change BugCount to be a property.

        //4) Increment code to see if after inserting a bug, it is stored within the repository
        [Test]
        public void Contains_InsertedBug_ReturnsTrue()
        {
            Repository repository = new Repository();
            Bug b = new Bug();
            repository.InsertBug(b); //need to insert the bug to see if it contains it
            Assert.IsTrue(repository.Contains(b), "Bug is not within repository");

        }

        //5) Implement contains (new List<Bug> - bugs.Contains())

        //6) Test fails. 

        //7) InsertBug - Add(bug)

        //8) Test passes 

        //9) Refactor - Bug Count could be based on collection

        #endregion

        #region Example 2 - Get single bug

        [Test]
        public void Single_Contains1Bug_BugReturned()
        {
            Repository repository = new Repository();
            Bug expected = new Bug();
            expected.ID = 1;
            repository.InsertBug(expected); //need to insert the bug to see if it contains it
            Bug actual = repository.Single(b => b.ID == 1);
            Assert.AreEqual(expected, actual);
        }

        #endregion

        #region Example 3 - Get All Active Bugs

        [Test]
        public void Where_TwoBugs_TwoBugsInIQueryable()
        {
            Repository repository = new Repository();
            Bug expected = new Bug();
            expected.IsActive = true;
            repository.InsertBug(expected);

            Bug expected2 = new Bug();
            expected2.IsActive = true;
            repository.InsertBug(expected2);

            IQueryable<Bug> actual = repository.Where(b => b.IsActive);
            Assert.AreEqual(2, actual.Count());
        }
        #endregion
    }
}
