﻿using System.Collections.Generic;
using System.Linq;
using NUnit.Framework;
using Rhino.Mocks;

namespace API_WithLinq.Tests
{
    [TestFixture]
    public class BugControllerTests
    {
        private BugController controller;
        private StubEmailService stubEmailService;
        private MockRepository mocks;
        IRepository repository;

        [SetUp]
        public void Setup()
        {
            mocks  = new MockRepository();
            repository = mocks.CreateMock<IRepository>();
            stubEmailService = new StubEmailService();
            controller = new BugController(repository, stubEmailService);
        }

        [Test]
        public void GetBugByID_ValidID_ReturnsBugObject()
        {
            int expectedBugID = 1;
            Bug insertedBug = new Bug();
            insertedBug.ID = expectedBugID;

            using (mocks.Record())
                Expect.Call(repository.Single(b => b.ID == 1))
                      .IgnoreArguments()//Because of Lambda expression
                      .Return(insertedBug);

            using (mocks.Playback())
            {
                Bug actual = controller.GetBugByID(expectedBugID);
                Assert.AreEqual(insertedBug.ID, actual.ID);
            }
        }

        [Test]
        public void InsertBug_ValidBugObject_ReturnsTrue()
        {
            Bug b = new Bug();

            using (mocks.Record())
            {
                Expect.Call(delegate { repository.InsertBug(b); });
                Expect.Call(repository.Contains(b)).Return(true);
            }

            using (mocks.Playback())
            {
                bool result = controller.InsertBug(b);
                Assert.IsTrue(result);
            }
        }

        [Test]
        public void GetAllActiveBugs_State_ExpectedBehaviour()
        {
            // I Love C# 3.0!
            List<Bug> bugs = new List<Bug>
                                 {
                                     new Bug
                                         {
                                             ID = 1,
                                             IsActive = true
                                         },
                                     new Bug
                                         {
                                             ID = 2,
                                             IsActive = true
                                         }
                                 };

            using (mocks.Record())
                Expect.Call(repository.Where(b => b.IsActive))
                      .IgnoreArguments() //Because of Lambda expression
                      .Return(bugs.AsQueryable()); //Strongly Typed

            using (mocks.Playback())
            {
                List<Bug> actualBugList = controller.GetAllActiveBugs();
                Assert.AreEqual(2, actualBugList.Count);
            }
        }
    }
}