﻿using System.Linq;
using NUnit.Framework;

namespace API_WithLinq.Tests
{
    [TestFixture]
    [Category("Integration")]
    public class RepositoryTests
    {
        #region Example 1 - Insert Bug

        [Test]
        public void InsertBug_ValidBugObject_CountIncremented()
        {
            Repository repository = new Repository();
            repository.InsertBug(new Bug());
            Assert.AreEqual(1, repository.BugCount);
        }

        [Test]
        public void Contains_InsertedBug_ReturnsTrue()
        {
            Repository repository = new Repository();
            Bug b = new Bug();
            repository.InsertBug(b);
            Assert.IsTrue(repository.Contains(b), "Bug is not within repository");
        }

        #endregion

        #region Example 2 - Get single bug

        [Test]
        public void Single_Contains1Bug_BugReturned()
        {
            Repository repository = new Repository();
            Bug expected = new Bug();
            expected.ID = 1;
            repository.InsertBug(expected); //need to insert the bug to see if it contains it
            Bug actual = repository.Single(b => b.ID == 1);
            Assert.AreEqual(expected, actual, "Expected: {0}, but was {1}", expected.ID, actual.ID);
        }

        #endregion

        #region Example 3 - Get All Active Bugs

        [Test]
        public void Where_TwoBugs_TwoBugsInIQueryable()
        {
            Repository repository = new Repository();
            Bug expected = new Bug();
            expected.IsActive = true;
            repository.InsertBug(expected);

            Bug expected2 = new Bug();
            expected2.IsActive = true;
            repository.InsertBug(expected2);

            IQueryable<Bug> actual = repository.Where(b => b.IsActive);
            Assert.AreEqual(2, Queryable.Count(actual));
        }
        #endregion

        #region ERROR

        /* After execute test it failed randomly:
         * 
            TestCase 'API_WithLinq.Tests.RepositoryTests.Single_Contains1Bug_BugReturned'
            failed: 
              Expected: 2, but was 1
              Expected: <API_WithLinq.Bug>
              But was:  <API_WithLinq.Bug>
        */

        //Reason was because we wasn't resetting the database correctly.  AutoID got us.
        [TearDown]
        public void Teardown()
        {
            DatabaseDataContext dc = new DatabaseDataContext();
            dc.Bugs.DeleteAllOnSubmit(dc.Bugs);
            dc.SubmitChanges();
            dc.ExecuteCommand("DBCC CHECKIDENT (Bug, RESEED, 0)", new object[] { });
        }

        //All fixed!

        #endregion
    }
}