﻿using System;
using System.Data.Linq;
using System.Linq;
using System.Linq.Expressions;

namespace API_WithLinq
{
    public interface IRepository
    {
        IQueryable<Bug> Where(Expression<Func<Bug, bool>> func);
        int BugCount { get; }
        void InsertBug(Bug bug);
        bool Contains(Bug b);
        Bug Single(Func<Bug, bool> func);
    }

    public class Repository : IRepository
    {
        public int BugCount { get { return Bugs.Count(); } }

        private DatabaseDataContext db;

        public Repository()
        {
            db = new DatabaseDataContext();
        }

        private Table<Bug> Bugs
        {
            get { return db.GetTable<Bug>(); }
        }

        public void InsertBug(Bug bug)
        {
            Bugs.InsertOnSubmit(bug);
            db.SubmitChanges();
        }

        public bool Contains(Bug b)
        {
            return Bugs.Contains(b);
        }

        public Bug Single(Func<Bug, bool> func)
        {
            return Bugs.Single(func);
        }

        public IQueryable<Bug> Where(Expression<Func<Bug, bool>> func)
        {
            return Bugs.Where(func);
        }
    }
}